#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <i86.h>
#include <mem.h>
#include <ctype.h>
#include <time.h>
#include "wave32.h"
#include "pmidi32.h"
#include "vesa32.h"

void DoExit( void )
{
    ShutDownWAVE();
    ShutDownMIDI();
}

#define BIOSseg 0x40

unsigned short *HEADptr = ((BIOSseg << 4) + 0x1a);
unsigned short *TAILptr = ((BIOSseg << 4) + 0x1c);
#define HEAD (*HEADptr)
#define TAIL (*TAILptr)

#define bios_kbhit() (HEAD - TAIL)

void _VideoPrint( char *s, int r, int c )
{
    char *ptr = (char*)(0xb8000 + 160 * r + c*2);
    int i = 0;

    while ( s[i] ) {
        *ptr++ = s[i++];
        *ptr++ = 0x3f;
    }
}

char scrmem[80*50*2];
int DrawCount = 0, DrawCountSave;
#define DRAW_RATE 12

void TIMER_CB_FUNC DrawScreen( void )
{
    u32 *p = (u32*)scrmem;
    int i;
    u32 v;
    byte c;
    static int ci = 0;
    static byte turn[] = "\\|/-";

    c = turn[ci++];
    DrawCount ++;

    v = 0x6C000000 | ( c << 16 ) | 0x6C00 | c;

    for( i=0; i < (80*50*2)/(4); i ++ ) {
        *p++ = v;
    }
    memcpy( (char*)0xb8000, scrmem, 80*50*2 );
    if ( ci > 3 ) ci = 0;
}

void ClrScr( void )
{
    u32 *p = (u32*)0xb8000;
    int i;

    for( i = 0; i < 80*50*2; i ++ ) {
        *p++ = 0x07200720;
    }
}

main( int argc, char **argv )
{
    int X, Y, MidiCount;
    int ArgI, I, Key, Quit, MaxWaves;
    int Volume, DrawScr;
    time_t time_beg, time_end;

    wave Waves[10];
    char buf[50];

    if ( argc < 3 ) {
        printf("usage: midwav [-] file.mid (list of waves)\n");
        printf("   If the '-' is given, the program will fill the screen\n");
        printf("   continuously with changing characters %d times/sec.\n",
                DRAW_RATE);

		return 0;
	}
    atexit(DoExit);     // deinitialization routine

    // load midi file given as the first argument

    ArgI = 1;

    DrawScr = 0;

    if ( argv[ArgI][0] == '-' ) {
        DrawScr = 1;
        ArgI ++;
    }

    if ( LoadMIDIFile( argv[ArgI] ) == 0 ) {
        printf("Error: Load midi file %s failed.\n", argv[ArgI] );
        exit(-2);
	}
    printf("Midi loaded.\n");

    // load each of the wave file given in the rest of the arguments

    ArgI ++;

    MaxWaves = 0;

    for( ; ArgI < argc; ArgI ++ ) {

        // Initialize wave memory pointer since there is no memory associate
        // with the pointer and LoadWave will try to free the memory if the
        // pointer is not NULL.

        Waves[MaxWaves].ptr = 0;

        if ( LoadWave( argv[ArgI], &Waves[MaxWaves] ) == 0 ) {
            printf("Loading wave %s failed.\n", argv[ArgI]);
            exit(-2);
        }
        printf("Load wave %d => %s.\n", MaxWaves, argv[ArgI]);
        MaxWaves ++;
        if ( MaxWaves == 10 ) break;
    }
    printf("\n%d Waves loaded.\n\n", MaxWaves );

    printf("While playing:\n");
    printf("Press the arrow keys to move the status display.\n");
    printf("Press '0'-'9' to play the corresponding wave.\n");
    printf("Press 'a' to play sound 0 at half of current volume.\n");
    printf("Press 'r' to remove all the sounds playing.\n\n");
    printf("Press 'u' to increase volume.\n");
    printf("Press 'd' to decrease volume.\n");
    printf("Press 's' to half the volume.\n");
    printf("Press 'f' to double the volume.\n");
    printf("Press 'b' to make volume 25%%.\n");
    printf("Press 't' to make volume 100%%.\n\n");
    printf("And press ESC to quit the program.\n");
    printf("\nPress a key to continue.\n\n");

    getch();

    // initialize the wave driver

    if ( PowerUpWAVE() == -1 ) {
        printf("Error: WAVE power up failed.\n");
        exit(-1);
    }

    // initialize the midi driver

	if ( PowerUpMIDI() == -1 ) {
		printf("Error: MIDI power up failed.\n");
        exit(-1);
	}

    // setup the wave playback parameters

    SetToWaveParam( &Waves[0] );

    // start playing midi

	PlayMIDI();

    // wait until ESC is pressed or the midi is finished.

    if ( DrawScr != 0 ) {
        DrawCount = 0;
        time_beg = time( NULL );
        VESARegisterTimer32( 230, DrawScreen, VESARateToDivisor32(DRAW_RATE));
    }

    X = 0;
    Y = 20;
    MidiCount = 0;
    Key = 0;
    Quit = 0;
    Volume = MAX_VOL_PERCENT/2;

    while( !Quit && MIDIPlaying ) {
        if ( bios_kbhit()) {

            _disable();
            Key = getch();
            if ( Key == 0 ) Key = getch() + 0xff;
            _enable();

            switch( Key ) {
                case 27:
                    Quit = 1; break;

                // move the midi status output location

                case 327: Y --; if ( Y < 0 ) Y = 49; break;
                case 330: X --; if ( X < 0 ) X = 79; break;
                case 332: X ++; if ( X > 79 ) X = 0; break;
                case 335: Y ++; if ( Y > 49) Y = 0; break;

                case 'u':
                case 'U':
                    if ( Volume < MAX_VOL_PERCENT ) Volume ++;
                    break;

                case 'd':
                case 'D':
                    if ( Volume > 1 ) Volume --;
                    break;

                // stop all playing waves

                case 'r':
                case 'R':
                // Remove all the sounds playing
                    for( I = 0; I < MIX_SNDS; I ++ ) {
                        RemoveSound(I);
                    }
                    break;

                case 's':
                    Volume /= 2;
                    break;

                case 'f':
                    Volume *= 2;
                    break;

                case 't':
                    Volume = MAX_VOL_PERCENT;
                    break;

                case 'b':
                    Volume = MAX_VOL_PERCENT/4;
                    break;


                // play a wave

                case 'a':
                    AddSound( &Waves[0], 0, Volume/2, 0, 0, 0 );
                    break;

                default:
                    if ( isdigit(Key)) {
                        I = Key - '0';
                        if ( I < MaxWaves ) {
                            AddSound( &Waves[I], 0, Volume, 0, 0, 0 );
                        }
                    }
                    break;
            }
        }
        // show midi playing status
        if ( MidiCount < MIDICallBackCount ) {
            MidiCount ++;
            sprintf( buf, "%d  Wave Volume: %d%%  ", MidiCount, Volume );
            _VideoPrint( buf, Y, X );
        }
	}
    if ( DrawScr ) {
        int TrueRate;

        time_end = time( NULL );
        DrawCountSave = DrawCount;
        VESARegisterTimer32( 230, 0, 0 );
        ClrScr();
        time_end -= time_beg;
        time_end ++;
        TrueRate = DrawCountSave/time_end;
        printf("time: %d  Count: %d  rate: %d/sec w/ %d extra frames\n",
                time_end, DrawCountSave, TrueRate,
                DrawCountSave%TrueRate);
    }

    return 0;
}
