;   /*\
;---|*|--------------------====< VESA Sound Include >====--------------------
;---|*|
;---|*| Copyright (c) 1993,1994  V.E.S.A, Inc. All Rights Reserved.
;---|*|
;---|*| VBE/AI 1.0 Specification
;---|*|    February 2, 1994. 1.00 release
;---|*|
;   \*/

#ifndef _VESA32_H_
#define _VESA32_H_

    #include <stdlib.h>
    #include "type32.h"
    #include "vbeai32.h"

;   /*\
;---|*|----====< VESA data types >====----
;   \*/

    // handle to the device

    typedef int VESAHANDLE;
    typedef int BLOCKHANDLE;

#define PASCAL              pascal
#define TRUE                -1
#define FALSE               0
#define ON                  TRUE
#define OFF                 FALSE
#define EXTERN              extern
#define PUBLIC
#define PRIVATE             static

typedef struct {
    long EDI;
    long ESI;
    long EBP;
    long reserved_by_system;
    long EBX;
    long EDX;
    long ECX;
    long EAX;
    i16  flags;
    i16  ES,DS,FS,GS,IP,CS,SP,SS;
} RMINFO;

;   /*\
;---|*| VESA high level calls
;   \*/



    PUBLIC VESAHANDLE PASCAL VESAFindADevice32( int );
    PUBLIC long  PASCAL VESAQueryDevice32( VESAHANDLE, i16, void * );
    PUBLIC void* PASCAL VESAOpenADevice32( VESAHANDLE, int, u16 );
    PUBLIC void  PASCAL VESACloseDevice32( int );

    PUBLIC int   PASCAL VESAFreeTimers32( void );
    PUBLIC int   PASCAL VESARegisterTimer32( VESAHANDLE, void (*)(), i32);
    PUBLIC i32   PASCAL VESARateToDivisor32( i32 );

    PUBLIC int   PASCAL VESAFreePatchBank32( VESAHANDLE );
    PUBLIC int   PASCAL VESAPreLoadPatch32( VESAHANDLE, int, int);
    PUBLIC int   PASCAL VESALoadPatchBank32( VESAHANDLE, fpMIDServ, char *);

    PUBLIC int   PASCAL DPMIAllocDosMem( size_t, u16 *, i16 * );
    PUBLIC int   PASCAL DPMIFreeDosMem( i16 );

    PUBLIC void PASCAL DPMISetPMVect( int, _int_isr );
    PUBLIC void PASCAL DPMISetRMVect( int, unsigned *);
    PUBLIC _int_isr PASCAL DPMIGetPMVect( int );
    PUBLIC unsigned PASCAL DPMIGetRMVect( int );

    /* Call real mode routine stuff
     * This implementation only allow one real mode routine to be
     * call at any time. */

    // add a real mode routine to call.  Returns a handle to the routine.
    PUBLIC int PASCAL AddRMRoutine( void * );

    // call a real mode routine using its handle

    PUBLIC int PASCAL CallReal( int );

    // non-zero if calling real mode in progress.

    extern int CRLock;


    // provide a stack to pass arguments to the real mode routine.
    // caller does not need to pop arguments off the stack.

    extern char *RMStackPtr;
    extern union REGS CRMregs;
    extern struct SREGS CRMsregs;

#define push( t, o )  push ## t ( o )
#define pop(t,o)  pop ## t(o)

#define pushlong(o) { RMStackPtr -= sizeof(long); *((long*)RMStackPtr) = (long)o; }
#define pushshort(o) { RMStackPtr -= sizeof(short); *((short*)RMStackPtr) = (short)o; }
#define pushu16(o) { RMStackPtr -= sizeof(u16); *((u16*)RMStackPtr) = (u16)o; }
#define pushi16(o) { RMStackPtr -= sizeof(i16); *((i16*)RMStackPtr) = (i16)o; }
#define pushu32(o) { RMStackPtr -= sizeof(u32); *((u32*)RMStackPtr) = (u32)o; }
#define pushi32(o) { RMStackPtr -= sizeof(i32); *((u32*)RMStackPtr) = (i32)o; }
#define pushvoidptr(o) { RMStackPtr -= sizeof(void*); *((void**)RMStackPtr) = (void*)o; }
#define pushptr(o) { RMStackPtr -= sizeof(char*); *((char**)RMStackPtr) = (char*)o; }
#define pushchar(o) { RMStackPtr -= sizeof(char); *((char*)RMStackPtr) = (char)o; }
#define pushbyte(o) { RMStackPtr -= sizeof(byte); *((byte*)RMStackPtr) = (byte)o; }
#define pushint(o) pushi32(o)

#define poplong(o) { o = (long)*((long*)RMStackPtr); RMStackPtr += sizeof(long); }
#define popshort(o) { o = (short)*((short*)RMStackPtr); RMStackPtr += sizeof(short); }
#define popu16(o) { o = (u16)*((u16*)RMStackPtr); RMStackPtr += sizeof(u16); }
#define popi16(o) { o = (i16)*((i16*)RMStackPtr); RMStackPtr += sizeof(i16); }
#define popu32(o) { o = (u32)*((u32*)RMStackPtr); RMStackPtr += sizeof(u32); }
#define popi32(o) { o = (i32)*((i32*)RMStackPtr); RMStackPtr += sizeof(i32); }
#define popvoidptr(o) { o = (void*)*((void**)RMStackPtr); RMStackPtr += sizeof(void*); }
#define popptr(o) { o = (char*)*((char**)RMStackPtr); RMStackPtr += sizeof(char*); }
#define popchar(o) { o = (char)*((char*)RMStackPtr); RMStackPtr += sizeof(char); }
#define popbyte(o) { o = (byte)*((byte*)RMStackPtr); RMStackPtr += sizeof(byte); }
#define popint(o) popi32(o)

#endif

;   /*\
;---|*| end of VESA.H
;   \*/

