#ifndef _WAVE_H_
#define _WAVE_H_

#define VALIDWAVE           0
#define UNKNOWNFILETYPE     1
#define MIX_LEN             (16*8)
#define MIX_SNDS            32
#define VERBOSE_MODE        0   /* Prints out some messages */

#define BOUNCE_BACK 10
#define WRAP_AROUND 20
#define VOLUMES 100
#define AMP     1
#define MAX_VOL_PERCENT (AMP*100)

#define MAX_VOLUMES (VOLUMES*AMP)

#include "type32.h"
#include "wavtyp32.h"

;   /*\
;---|*| public prototypes
;   \*/

    PUBLIC int  PASCAL PowerUpWAVE            ( void );
//
// Return zero if failed, else returns positive non-zero
//

    PUBLIC void PASCAL ShutDownWAVE           ( void );
//
// It's ok to call this a million times since it makes sure that the WAVE
// driver is up before shutting it down.
//

	PUBLIC int PASCAL  SetToWaveParam( wave *Wav );
//
// Use the parameters from one wave file to play all the sounds.
// returns non-zero if succeed or zero for failure.
//

	PUBLIC int PASCAL  LoadWave( char *, wave * );
//
// Load a wave file into the struct pointer's data.  If the data pointer
// of the struct is not zero, then LoadWave will try to free up the memory
// assuming it's been allocated.  Returns positive non-zero if succeed,
// else returns 0.
//

    PUBLIC int PASCAL  AddSound( wave *,
                                    int rep, int vol, int vr, int vc, int ff );
//
// Add a wave to the mixing list.  repeat the sound if rep is non-zero.
// Returns the index of the slot the wave was assigned or -1 if the
// no more sounds can be added to the mix list.
//

    PUBLIC int PASCAL  RemoveSound( int which );
//
// Remove a playing sound from the list of waves being play.
// returns 0 if failed else returns 1.
//

// protected mode interface to the services provided by the driver

    PUBLIC int PASCAL wsWaveRegister( char *ptr, i32 len );
    PUBLIC int PASCAL wsPlayBlock( i16 han, i32 fill );
    PUBLIC int PASCAL wsGetLastError( void );
    PUBLIC int PASCAL wsPlayCont( void *ptr, i32 Len, i32 LenDiv );
    PUBLIC int PASCAL wsPCMInfo( i16 ch, i32 sr, i16 cp, i16 un, i16 sz );
    PUBLIC int PASCAL wsStopIO( i16 a );


;   /*\
;---|*| PUBLIC Global variables
;   \*/

    /* This is a read-only variable
     * Changing it won't affect the WAVE mixing and playing driver
     * but your own codes */
    EXTERN int SoundPlaying;

#endif
