#ifndef _WAVE_TYPE_H
#define _WAVE_TYPE_H

#include "type32.h"

typedef unsigned char byte;

    typedef struct {
        char *fname;        // file name
        signed char *ptr;   // pointer to the buffer
        i32 len;            // file length
        i32 han;           // registered handle
        i32 sr;             // sample rate
        i16 ch;            // channels
        i16 sz;            // data size (8 or 16)
        i16 cp;            // compression
    } wave;

// 4. Wave format control block

    typedef struct {
        i16  formatTag;     // format category
        i16  nChannels;     // stereo/mono
        i32 nSamplesPerSec;    // sample rate
        i32 nAvgBytesPerSec;   // stereo * sample rate
        i16  nBlockAlign;       // block alignment (1=byte)
        i16  nBitsPerSample;    // # byte bits per sample
    } WaveInfo;

// 3. Wave detailed information Block

    typedef struct {
        char name[4];   // "fmt "
        i32 length;
        WaveInfo info;
    } WaveFormat;

// 3. Data header which follows a WaveFormat Block

    typedef struct {
        char name[4];   // "data"
        i32 length;
    } DataHeader;

// 2. Total Wave Header data in a wave file

    typedef struct {
        char name[4];   // "WAVE"
        WaveFormat fmt;
        DataHeader data;
    } WaveHeader;

// 2. Riff wrapper around the WaveFormat Block (optional)

    typedef struct {
        char name[4];   // "RIFF"
        i32 length;
    } RiffHeader;

// 1. Riff wrapped WaveFormat Block

    typedef struct {
        RiffHeader riff;
        WaveHeader wave;
    } RiffWave;

// 0. A sound effect for mixing output

    typedef struct {
        char *ptr;
        int valid;
        int repeat;
        i32 len;
        i32 pos;
        int volume;
        int VolRate;
        int VolCount;
        int VolChange;
        int FlipFlop;
    } Sound;

#endif
